#ifndef ALPHABET_HH_
#define ALPHABET_HH_


#include <vector>
#include <string>
#include <map>

namespace biu
{

		/*! This class handels an alphabet for the sequence.
		 *
		 *  Each member of the alphabet maps to an unique int index. 
		 *
		 * @author Martin Mann, Sebastian Will, Andreas Richter
		 */
	class Alphabet
	{
	public:
			//! The internal representation of an alphabet element.
		typedef unsigned int	AlphElem;
			//! The internal representation of a compressed alphabet element.
		typedef unsigned char	CAlphElem;
		
			//! The internal representation of a sequence of alphabet elements.
		typedef std::vector<AlphElem> Sequence;
		
			//! The internal representation of a compressed sequence of alphabet elements.
		typedef std::vector<CAlphElem> CSequence;
		
	private:
	
			//! just a shortcut
		typedef std::map<std::string, AlphElem> STR2ALPH_TYPE;
		
			//! a mapping from a string representation to the corresponding 
			//! alphabet element
		STR2ALPH_TYPE string2alph;	
		
			//! a mapping from alphabet elements to their string representation
		std::vector<std::string> alph2string;	
		
			//! the length of a string representation of an alphabet element.
		unsigned int elementLength;		
		
	public:
	
			/*! Initialises an Alphabet object with the string representations 
			 * given by the alphabetString. The string is members given in 
			 * alphabetString.
			 * 
			 * The string it sliced into substrings of the length elementLength_
			 * , therefor all alphabet elements have to have the same length.
			 */
		Alphabet(	const std::string& alphabetString, 
					const unsigned int elementLength_);
		
			/*! Construct an alphabet from a vector of strings
			*
			* The indexing of elements in the vector has to be preserved by
			* the alphabet
			*/
		Alphabet(const std::vector<std::string> & alphabetStrings);
		
		virtual ~Alphabet();

		bool operator== (const Alphabet& alph2) const;
		bool operator!= (const Alphabet& alph2) const;

			//! Returns the size of the alphabet. 
		size_t getAlphabetSize() const {
			return alph2string.size();
		}
		
			//! Returns the length of a alphabet element.
		unsigned int getElementLength() const { 
			return elementLength;
		}
	
		//////////////////////////////////////////////////	
		// string to sequence to string
		//////////////////////////////////////////////////
	
			//! Returns an internal sequence representation of the string.
		Alphabet::Sequence getSequence(const std::string& seqString) const;
		
			//! Returns the index the alphabet member maps to.
		Alphabet::AlphElem getElement(const std::string& alphElemStr) const;
		
			//! Converts an internal sequence representation into a string.
		std::string	getString(const Alphabet::Sequence& sequence) const;
		
			//! Returns the string representation of the alphabet member.
		std::string getString(const Alphabet::AlphElem& elem) const;
		
			//! Compresses a sequence.
		Alphabet::CSequence compress(const Alphabet::Sequence& sequence) const;
		
			//! Decompresses a sequence.
			//! @param sequence		the sequence to decompress
			//! @param seqLength 	the length of the uncompressed sequence encoded 
		Sequence decompress(const CSequence& sequence, const size_t seqLength) const;
		
	
		//////////////////////////////////////////////////
		// miscellaneous
		//////////////////////////////////////////////////
	
			//! Returns whether or not a string contains only elements
			//! of the alphabet or not
		bool isAlphabetString(const std::string& str) const;
		
			//! Returns whether or not a sequencec contains only elements
			//! compatible with the alphabet or not
		bool isAlphabetSequence(const Sequence& seq) const;
		
			//! Returns the internal index of the AlphElem. The index
			//! starts with 0 and is < getAlphabetSize()
		unsigned int getIndex(const AlphElem& elem) const;
		
			//! Returns the internal index of the AlphElem corresponding
			//! to the string. The index
			//! starts with 0 and is < getAlphabetSize()
		unsigned int getIndex(const std::string& elemStr) const;
		
			//! Returns the corresponding AlphElem to the index.
			//! @param index has to be < getAlphabetSize()
		AlphElem getElement(const unsigned int index) const;
		
	};

} // namespace biu

#endif /*ALPHABET_HH_*/
